from super_gradients.training.models import ResNeXt50, ResNeXt101, googlenet_v1
from super_gradients.training.models.classification_models import repvgg, efficientnet, densenet, resnet, regnet
from super_gradients.training.models.classification_models.mobilenetv2 import mobile_net_v2, mobile_net_v2_135, \
    custom_mobile_net_v2
from super_gradients.training.models.classification_models.mobilenetv3 import mobilenetv3_large, mobilenetv3_small, \
    mobilenetv3_custom
from super_gradients.training.models.classification_models.shufflenetv2 import ShufflenetV2_x0_5, ShufflenetV2_x1_0, \
    ShufflenetV2_x1_5, \
    ShufflenetV2_x2_0, CustomizedShuffleNetV2
from super_gradients.training.models.classification_models.vit import vit_base, vit_large, vit_huge
from super_gradients.training.models.detection_models.csp_darknet53 import CSPDarknet53
from super_gradients.training.models.detection_models.darknet53 import Darknet53
from super_gradients.training.models.detection_models.ssd import SSDMobileNetV1, SSDLiteMobileNetV2
from super_gradients.training.models.detection_models.yolox import YoloX_N, YoloX_T, YoloX_S, YoloX_M, YoloX_L, YoloX_X, CustomYoloX
from super_gradients.training.models.segmentation_models.ddrnet import DDRNet23, DDRNet23Slim, AnyBackBoneDDRNet23
from super_gradients.training.models.segmentation_models.regseg import RegSeg48
from super_gradients.training.models.segmentation_models.shelfnet import ShelfNet18_LW, ShelfNet34_LW, ShelfNet50, \
    ShelfNet503343, ShelfNet101
from super_gradients.training.models.segmentation_models.stdc import STDC1Classification, STDC2Classification, \
    STDC1Seg, STDC2Seg

from super_gradients.training.models.kd_modules.kd_module import KDModule
from super_gradients.training.models.classification_models.beit import beit_base_patch16_224, beit_large_patch16_224
from super_gradients.training.models.segmentation_models.ppliteseg import PPLiteSegT, PPLiteSegB
# IF YOU UPDATE THE ARCHITECTURE DICT PLEASE ALSO UPDATE THE ENUM CLASS DOWN BELOW.


ARCHITECTURES = {"resnet18": resnet.ResNet18,
                 "resnet34": resnet.ResNet34,
                 "resnet50_3343": resnet.ResNet50_3343,
                 "resnet50": resnet.ResNet50,
                 "resnet101": resnet.ResNet101,
                 "resnet152": resnet.ResNet152,
                 "resnet18_cifar": resnet.ResNet18Cifar,
                 "custom_resnet": resnet.CustomizedResnet,
                 "custom_resnet50": resnet.CustomizedResnet50,
                 "custom_resnet_cifar": resnet.CustomizedResnetCifar,
                 "custom_resnet50_cifar": resnet.CustomizedResnet50Cifar,
                 "mobilenet_v2": mobile_net_v2,
                 "mobile_net_v2_135": mobile_net_v2_135,
                 "custom_mobilenet_v2": custom_mobile_net_v2,
                 "mobilenet_v3_large": mobilenetv3_large,
                 "mobilenet_v3_small": mobilenetv3_small,
                 "mobilenet_v3_custom": mobilenetv3_custom,
                 "custom_densenet": densenet.CustomizedDensnet,
                 "densenet121": densenet.densenet121,
                 "densenet161": densenet.densenet161,
                 "densenet169": densenet.densenet169,
                 "densenet201": densenet.densenet201,
                 "shelfnet18_lw": ShelfNet18_LW,
                 "shelfnet34_lw": ShelfNet34_LW,
                 "shelfnet50_3343": ShelfNet503343,
                 "shelfnet50": ShelfNet50,
                 "shelfnet101": ShelfNet101,
                 "shufflenet_v2_x0_5": ShufflenetV2_x0_5,
                 "shufflenet_v2_x1_0": ShufflenetV2_x1_0,
                 "shufflenet_v2_x1_5": ShufflenetV2_x1_5,
                 "shufflenet_v2_x2_0": ShufflenetV2_x2_0,
                 "shufflenet_v2_custom5": CustomizedShuffleNetV2,
                 'darknet53': Darknet53,
                 'csp_darknet53': CSPDarknet53,
                 "resnext50": ResNeXt50,
                 "resnext101": ResNeXt101,
                 "googlenet_v1": googlenet_v1,
                 "efficientnet_b0": efficientnet.b0,
                 "efficientnet_b1": efficientnet.b1,
                 "efficientnet_b2": efficientnet.b2,
                 "efficientnet_b3": efficientnet.b3,
                 "efficientnet_b4": efficientnet.b4,
                 "efficientnet_b5": efficientnet.b5,
                 "efficientnet_b6": efficientnet.b6,
                 "efficientnet_b7": efficientnet.b7,
                 "efficientnet_b8": efficientnet.b8,
                 "efficientnet_l2": efficientnet.l2,
                 "CustomizedEfficientnet": efficientnet.CustomizedEfficientnet,
                 "regnetY200": regnet.RegNetY200,
                 "regnetY400": regnet.RegNetY400,
                 "regnetY600": regnet.RegNetY600,
                 "regnetY800": regnet.RegNetY800,
                 "custom_regnet": regnet.CustomRegNet,
                 "custom_anynet": regnet.CustomAnyNet,
                 "nas_regnet": regnet.NASRegNet,
                 "yolox_n": YoloX_N,
                 "yolox_t": YoloX_T,
                 "yolox_s": YoloX_S,
                 "yolox_m": YoloX_M,
                 "yolox_l": YoloX_L,
                 "yolox_x": YoloX_X,
                 "custom_yolox": CustomYoloX,
                 "ssd_mobilenet_v1": SSDMobileNetV1,
                 "ssd_lite_mobilenet_v2": SSDLiteMobileNetV2,
                 "repvgg_a0": repvgg.RepVggA0,
                 "repvgg_a1": repvgg.RepVggA1,
                 "repvgg_a2": repvgg.RepVggA2,
                 "repvgg_b0": repvgg.RepVggB0,
                 "repvgg_b1": repvgg.RepVggB1,
                 "repvgg_b2": repvgg.RepVggB2,
                 "repvgg_b3": repvgg.RepVggB3,
                 "repvgg_d2se": repvgg.RepVggD2SE,
                 "repvgg_custom": repvgg.RepVggCustom,
                 "ddrnet_23": DDRNet23,
                 "ddrnet_23_slim": DDRNet23Slim,
                 "custom_ddrnet_23": AnyBackBoneDDRNet23,
                 "stdc1_classification": STDC1Classification,
                 "stdc2_classification": STDC2Classification,
                 "stdc1_seg": STDC1Seg,
                 "stdc1_seg50": STDC1Seg,
                 "stdc1_seg75": STDC1Seg,
                 "stdc2_seg": STDC2Seg,
                 "stdc2_seg50": STDC2Seg,
                 "stdc2_seg75": STDC2Seg,
                 "regseg48": RegSeg48,
                 "kd_module": KDModule,
                 "vit_base": vit_base,
                 "vit_large": vit_large,
                 "vit_huge": vit_huge,
                 "beit_base_patch16_224": beit_base_patch16_224,
                 "beit_large_patch16_224": beit_large_patch16_224,
                 "pp_lite_t_seg": PPLiteSegT,
                 "pp_lite_t_seg50": PPLiteSegT,
                 "pp_lite_t_seg75": PPLiteSegT,
                 "pp_lite_b_seg": PPLiteSegB,
                 "pp_lite_b_seg50": PPLiteSegB,
                 "pp_lite_b_seg75": PPLiteSegB,
                 }

KD_ARCHITECTURES = {
    "kd_module": KDModule
}
